@@ -15,7 +15,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vfsops.c,v 1.11.2.5 2002/07/22 23:14:51 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vfsops.c,v 1.14 2002/01/29 17:26:26 kolya Exp $");
 
 #include "../afs/sysincludes.h"
 #include "../afs/afsincludes.h"
@@ -134,17 +134,17 @@ static int afs_root(struct super_block *afsp)
 #endif
 		
 		/* "/afs" is a directory, reset inode ops accordingly. */
-		AFSTOV(tvp)->v_op = &afs_dir_iops;
+		tvp->v.v_op = &afs_dir_iops;
 #if defined(AFS_LINUX24_ENV)
-		AFSTOV(tvp)->v_fop = &afs_dir_fops;
+		tvp->v.v_fop = &afs_dir_fops;
 #endif
 
 		/* setup super_block and mount point inode. */
 		afs_globalVp = tvp;
 #if defined(AFS_LINUX24_ENV)
-		afsp->s_root = d_alloc_root(AFSTOI(tvp));
+		afsp->s_root = d_alloc_root((struct inode*)&tvp->v);
 #else
-		afsp->s_root = d_alloc_root(AFSTOI(tvp), NULL);
+		afsp->s_root = d_alloc_root((struct inode*)tvp, NULL);
 #endif
 		afsp->s_root->d_op = &afs_dentry_operations;
 	    } else
@@ -188,8 +188,8 @@ int afs_notify_change(struct dentry *dp, struct iattr* iattrp)
     VATTR_NULL(&vattr);
     iattr2vattr(&vattr, iattrp); /* Convert for AFS vnodeops call. */
     update_inode_cache(ip, &vattr);
-    code = afs_setattr(ITOAFS(ip), &vattr, credp);
-    afs_CopyOutAttrs(ITOAFS(ip), &vattr);
+    code = afs_setattr((struct vcache*)ip, &vattr, credp);
+    afs_CopyOutAttrs((struct vcache*)ip, &vattr);
     /* Note that the inode may still not have all the correct info. But at
      * least we've got the newest version of what was supposed to be set.
      */
@@ -242,7 +242,7 @@ void afs_write_inode(struct inode *ip)
 
 void afs_delete_inode(struct inode *ip)
 {
-    struct vcache *vc = ITOAFS(ip);
+    struct vcache *vc = (struct vcache*)ip;
 
     AFS_GLOCK();
     osi_clear_inode(ip);
@@ -463,13 +463,3 @@ void vcache2inode(struct vcache *avc)
     VATTR_NULL(&vattr);
     afs_CopyOutAttrs(avc, &vattr); /* calls vattr2inode */
 }
-
-/* Yet another one for fakestat'ed mountpoints */
-void vcache2fakeinode(struct vcache *rootvp, struct vcache *mpvp)
-{
-    struct vattr vattr;
-
-    VATTR_NULL(&vattr);
-    afs_CopyOutAttrs(rootvp, &vattr);
-    vattr2inode(AFSTOV(mpvp), &vattr);
-}
