@@ -23,7 +23,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.29.2.25 2004/10/21 15:01:20 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.33 2002/01/16 02:08:43 shadow Exp $");
 
 #include "../afs/sysincludes.h"
 #include "../afs/afsincludes.h"
@@ -42,7 +42,6 @@ RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.29.2.25 2004/10/21
 #endif
 
 extern struct vcache *afs_globalVp;
-extern afs_rwlock_t afs_xvcache;
 
 extern struct dentry_operations *afs_dops;
 #if defined(AFS_LINUX24_ENV)
@@ -62,13 +61,13 @@ static ssize_t afs_linux_read(struct file *fp, char *buf, size_t count,
 			      loff_t *offp)
 {
     ssize_t code;
-    struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
+    struct vcache *vcp = (struct vcache*)fp->f_dentry->d_inode;
     cred_t *credp = crref();
     struct vrequest treq;
 
     AFS_GLOCK();
     afs_Trace4(afs_iclSetp, CM_TRACE_READOP, ICL_TYPE_POINTER, vcp,
-	       ICL_TYPE_INT32, (int)*offp,
+	       ICL_TYPE_OFFSET, offp,
 	       ICL_TYPE_INT32, count,
 	       ICL_TYPE_INT32, 99999);
 
@@ -80,14 +79,51 @@ static ssize_t afs_linux_read(struct file *fp, char *buf, size_t count,
     if (code)
 	code = -code;
     else {
-	osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
-	AFS_GUNLOCK();
-	code = generic_file_read(fp, buf, count, offp);
-	AFS_GLOCK();
+#ifdef AFS_64BIT_CLIENT
+	if (*offp + count > afs_vmMappingEnd) {
+    	    uio_t tuio;
+    	    struct iovec iov;
+	    afs_size_t oldOffset = *offp;
+	    afs_int32 xfered = 0;
+
+	    if (*offp < afs_vmMappingEnd) {
+		/* special case of a buffer crossing the VM mapping end */
+		afs_int32 tcount = afs_vmMappingEnd - *offp;
+		count -= tcount;
+		osi_FlushPages(vcp, credp); /* ensure stale pages are gone */
+		AFS_GUNLOCK();
+		code = generic_file_read(fp, buf, tcount, offp);
+		AFS_GLOCK();
+		if (code != tcount) {
+		    goto done;
+		}
+		xfered = tcount;
+	    } 
+            setup_uio(&tuio, &iov, buf + xfered, (afs_offs_t) *offp, count, 
+						UIO_READ, AFS_UIOSYS);
+            code = afs_read(vcp, &tuio, credp, 0, 0, 0);
+	    xfered += count - tuio.uio_resid;
+	    if (code != 0) {
+		code = xfered;
+		*offp += count - tuio.uio_resid;
+	    } else {
+		code = xfered;
+		*offp += count;
+	    }
+done:
+	} else {
+#endif /* AFS_64BIT_CLIENT */
+	    osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
+	    AFS_GUNLOCK();
+	    code = generic_file_read(fp, buf, count, offp);
+	    AFS_GLOCK();
+#ifdef AFS_64BIT_CLIENT
+	}
+#endif /* AFS_64BIT_CLIENT */
     }
 
     afs_Trace4(afs_iclSetp, CM_TRACE_READOP, ICL_TYPE_POINTER, vcp,
-	       ICL_TYPE_INT32, (int)*offp,
+	       ICL_TYPE_OFFSET, offp,
 	       ICL_TYPE_INT32, count,
 	       ICL_TYPE_INT32, code);
 
@@ -106,15 +142,16 @@ static ssize_t afs_linux_write(struct file *fp, const char *buf, size_t count,
 {
     ssize_t code = 0;
     int code2;
-    struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
+    struct vcache *vcp = (struct vcache *)fp->f_dentry->d_inode;
     struct vrequest treq;
     cred_t *credp = crref();
 
     AFS_GLOCK();
 
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITEOP, ICL_TYPE_POINTER, vcp,
-	       ICL_TYPE_INT32, (int)*offp, ICL_TYPE_INT32, count,
-	       ICL_TYPE_INT32, (fp->f_flags & O_APPEND) ? 99998 : 99999);
+	        ICL_TYPE_OFFSET, offp, 
+		ICL_TYPE_INT32, count,
+	        ICL_TYPE_INT32, (fp->f_flags & O_APPEND) ? 99998 : 99999);
 
 
     /* get a validated vcache entry */
@@ -125,26 +162,80 @@ static ssize_t afs_linux_write(struct file *fp, const char *buf, size_t count,
     ObtainWriteLock(&vcp->lock, 529);
     afs_FakeOpen(vcp);
     ReleaseWriteLock(&vcp->lock);
-    AFS_GUNLOCK();
     if (code)
 	code = -code;
     else {
-	code = generic_file_write(fp, buf, count, offp);
+#ifdef AFS_64BIT_CLIENT
+	if (*offp + count > afs_vmMappingEnd) {
+    	    uio_t tuio;
+    	    struct iovec iov;
+	    afs_size_t oldOffset = *offp;
+	    afs_int32 xfered = 0;
+
+	    if (*offp < afs_vmMappingEnd) {
+		/* special case of a buffer crossing the VM mapping end */
+		afs_int32 tcount = afs_vmMappingEnd - *offp;
+		count -= tcount;
+		AFS_GUNLOCK();
+		code = generic_file_write(fp, buf, tcount, offp);
+		AFS_GLOCK();
+		if (code != tcount) {
+		    goto done;
+		}
+		xfered = tcount;
+	    } 
+            setup_uio(&tuio, &iov, buf + xfered, (afs_offs_t) *offp, count, 
+						UIO_WRITE, AFS_UIOSYS);
+            code = afs_write(vcp, &tuio, fp->f_flags, credp, 0);
+	    xfered += count - tuio.uio_resid;
+	    if (code != 0) {
+		code = xfered;
+		*offp += count - tuio.uio_resid;
+	    } else {
+	        /* Purge dirty chunks of file if there are too many dirty chunks.
+         	* Inside the write loop, we only do this at a chunk boundary.
+         	* Clean up partial chunk if necessary at end of loop.
+         	*/
+        	if (AFS_CHUNKBASE(tuio.afsio_offset) != AFS_CHUNKBASE(oldOffset)) {
+            	    ObtainWriteLock(&vcp->lock,402);
+            	    code = afs_DoPartialWrite(vcp, &treq);
+            	    vcp->states |= CDirty;
+            	    ReleaseWriteLock(&vcp->lock);
+        	}
+		code = xfered;
+		*offp += count;
+		ObtainWriteLock(&vcp->lock,400);
+        	vcp->m.Date = osi_Time();       /* Set file date (for ranlib) */
+        	/* extend file */
+        	if (*offp > vcp->m.Length) {
+            	    vcp->m.Length = *offp;
+        	}
+        	ReleaseWriteLock(&vcp->lock);
+	    }
+done:
+	} else {
+#endif /* AFS_64BIT_CLIENT */
+            AFS_GUNLOCK();
+	    code = generic_file_write(fp, buf, count, offp);
+            AFS_GLOCK();
+#ifdef AFS_64BIT_CLIENT
+	}
+#endif /* AFS_64BIT_CLIENT */
     }
-    AFS_GLOCK();
 
     ObtainWriteLock(&vcp->lock, 530);
     vcp->m.Date = osi_Time(); /* set modification time */
     afs_FakeClose(vcp, credp);
     if (code>=0)
-	code2 = afs_DoPartialWrite(vcp, &treq);
+        code2 = afs_DoPartialWrite(vcp, &treq);
     if (code2 && code >=0)
-	code = (ssize_t) -code2;
+        code = (ssize_t) -code2;
     ReleaseWriteLock(&vcp->lock);
 	
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITEOP, ICL_TYPE_POINTER, vcp,
-	       ICL_TYPE_INT32, (int)*offp, ICL_TYPE_INT32, count,
-	       ICL_TYPE_INT32, code);
+	       	ICL_TYPE_OFFSET, offp, 
+		ICL_TYPE_INT32, count,
+	       	ICL_TYPE_INT32, code);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -159,7 +250,7 @@ static int afs_linux_readdir(struct file *fp,
 			     void *dirbuf, filldir_t filldir)
 {
     extern struct DirEntry * afs_dir_GetBlob();
-    struct vcache *avc = ITOAFS(FILE_INODE(fp));
+    struct vcache *avc = (struct vcache*)FILE_INODE(fp);
     struct vrequest treq;
     register struct dcache *tdc;
     int code;
@@ -168,9 +259,8 @@ static int afs_linux_readdir(struct file *fp,
     struct DirEntry *de;
     ino_t ino;
     int len;
-    int origOffset;
+    afs_size_t origOffset, tlen;
     cred_t *credp = crref();
-    struct afs_fakestat_state fakestat;
 
     AFS_GLOCK();
     AFS_STATCNT(afs_readdir);
@@ -182,31 +272,23 @@ static int afs_linux_readdir(struct file *fp,
 	return -code;
     }
 
-    afs_InitFakeStat(&fakestat);
-    code = afs_EvalFakeStat(&avc, &fakestat, &treq);
-    if (code) {
-	afs_PutFakeStat(&fakestat);
-	AFS_GUNLOCK();
-	return -code;
-    }
-
     /* update the cache entry */
 tagain:
     code = afs_VerifyVCache(avc, &treq);
     if (code) {
-	afs_PutFakeStat(&fakestat);
 	AFS_GUNLOCK();
 	return -code;
     }
 
     /* get a reference to the entire directory */
-    tdc = afs_GetDCache(avc, 0, &treq, &origOffset, &len, 1);
+    tdc = afs_GetDCache(avc, (afs_size_t) 0, &treq, &origOffset, &tlen, 1);
+    len = tlen;
     if (!tdc) {
-	afs_PutFakeStat(&fakestat);
 	AFS_GUNLOCK();
 	return -ENOENT;
     }
     ObtainReadLock(&avc->lock);
+    ObtainReadLock(&tdc->lock);
     /*
      * Make sure that the data in the cache is current. There are two
      * cases we need to worry about:
@@ -214,15 +296,17 @@ tagain:
      * 2. The cache data is no longer valid
      */
     while ((avc->states & CStatd)
-	   && (tdc->flags & DFFetching)
+	   && (tdc->dflags & DFFetching)
 	   && hsame(avc->m.DataVersion, tdc->f.versionNo)) {
-	tdc->flags |= DFWaiting;
+	ReleaseReadLock(&tdc->lock);
 	ReleaseReadLock(&avc->lock);
 	afs_osi_Sleep(&tdc->validPos);
 	ObtainReadLock(&avc->lock);
+	ObtainReadLock(&tdc->lock);
     }
     if (!(avc->states & CStatd)
 	|| !hsame(avc->m.DataVersion, tdc->f.versionNo)) {
+	ReleaseReadLock(&tdc->lock);
 	ReleaseReadLock(&avc->lock);
 	afs_PutDCache(tdc);
 	goto tagain;
@@ -259,12 +343,11 @@ tagain:
              afid.Fid.Unique=ntohl(de->fid.vunique);
              if ((avc->states & CForeign) == 0 &&
                  (ntohl(de->fid.vnode) & 1)) {
-                  type=DT_DIR;
              } else if ((tvc=afs_FindVCache(&afid,0,0,0,0))) {
                   if (tvc->mvstat) {
                        type=DT_DIR;
                   } else if (((tvc->states) & (CStatd|CTruth))) {
-                       /* CTruth will be set if the object has 
+                       /* CTruth will be set if the object has
                         *ever* been statd */
                        vtype=vType(tvc);
                        if (vtype == VDIR)
@@ -276,13 +359,13 @@ tagain:
                           type=DT_LNK; */
                        /* what other types does AFS support? */
                   }
-		  /* clean up from afs_FindVCache */
-		  afs_PutVCache(tvc, WRITE_LOCK);
+                  /* clean up from afs_FindVCache */
+                  afs_PutVCache(tvc, WRITE_LOCK);
              }
              code = (*filldir)(dirbuf, de->name, len, offset, ino, type);
         }
 #else
-	code = (*filldir)(dirbuf, de->name, len, offset, ino); 
+        code = (*filldir)(dirbuf, de->name, len, offset, ino);
 #endif
 	DRelease(de, 0);
 	if (code)
@@ -294,9 +377,9 @@ tagain:
      */
     fp->f_pos = (loff_t)offset;
 
+    ReleaseReadLock(&tdc->lock);
     afs_PutDCache(tdc);
     ReleaseReadLock(&avc->lock);
-    afs_PutFakeStat(&fakestat);
     AFS_GUNLOCK();
     return 0;
 }
@@ -328,7 +411,7 @@ void afs_linux_vma_close(struct vm_area_struct *vmap)
     if (!vmap->vm_file)
 	return;
 
-    vcp = ITOAFS(FILE_INODE(vmap->vm_file));
+    vcp = (struct vcache*)FILE_INODE(vmap->vm_file);
     if (!vcp)
 	return;
 
@@ -364,7 +447,7 @@ void afs_linux_vma_close(struct vm_area_struct *vmap)
 
 static int afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
 {
-    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
+    struct vcache *vcp = (struct vcache*)FILE_INODE(fp);
     cred_t *credp = crref();
     struct vrequest treq;
     int code;
@@ -372,8 +455,8 @@ static int afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
     AFS_GLOCK();
 #if defined(AFS_LINUX24_ENV)
     afs_Trace3(afs_iclSetp, CM_TRACE_GMAP, ICL_TYPE_POINTER, vcp,
-	       ICL_TYPE_POINTER, vmap->vm_start,
-	       ICL_TYPE_INT32, vmap->vm_end - vmap->vm_start);
+               ICL_TYPE_POINTER, vmap->vm_start,
+               ICL_TYPE_INT32, vmap->vm_end - vmap->vm_start);
 #else
     afs_Trace4(afs_iclSetp, CM_TRACE_GMAP, ICL_TYPE_POINTER, vcp,
 	       ICL_TYPE_POINTER, vmap->vm_start,
@@ -418,8 +501,8 @@ static int afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
 	    }
 	    vmap->vm_ops = &afs_private_mmap_ops;
 	}
-    
-    
+
+
 	/* Add an open reference on the first mapping. */
 	if (vcp->mapcnt == 0) {
 	    vcp->execsOrWriters++;
@@ -465,7 +548,7 @@ static int afs_linux_release(struct inode *ip, struct file *fp)
 {
     int code = 0;
     cred_t *credp = crref();
-    struct vcache *vcp = ITOAFS(ip);
+    struct vcache *vcp = (struct vcache*)ip;
 
     AFS_GLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -500,7 +583,7 @@ static int afs_linux_fsync(struct file *fp, struct dentry *dp)
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
 #endif
-    code = afs_fsync(ITOAFS(ip), credp);
+    code = afs_fsync((struct vcache*)ip, credp);
 #ifdef AFS_LINUX24_ENV
     unlock_kernel();
 #endif
@@ -524,7 +607,7 @@ int afs_linux_file_revalidate(kdev_t dev);
 static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
 {
     int code = 0;
-    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
+    struct vcache *vcp = (struct vcache*)FILE_INODE(fp);
     cred_t *credp = crref();
 #ifdef AFS_LINUX24_ENV
     struct flock64 flock;
@@ -540,26 +623,9 @@ static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
     flock.l_start = flp->fl_start;
     flock.l_len = flp->fl_end - flp->fl_start;
 
-    /* Safe because there are no large files, yet */
-#if defined(F_GETLK64) && (F_GETLK != F_GETLK64)
-    if (cmd == F_GETLK64)
-	cmd = F_GETLK;
-    else if (cmd == F_SETLK64)
-	cmd = F_SETLK;
-    else if (cmd == F_SETLKW64)
-	cmd = F_SETLKW;
-#endif /* F_GETLK64 && F_GETLK != F_GETLK64 */
-
     AFS_GLOCK();
     code = afs_lockctl(vcp, &flock, cmd, credp);
     AFS_GUNLOCK();
-
-    /* Convert flock back to Linux's file_lock */
-    flp->fl_type = flock.l_type;
-    flp->fl_pid = flock.l_pid;
-    flp->fl_start = flock.l_start;
-    flp->fl_end = flock.l_start + flock.l_len;
-
     crfree(credp);
     return -code;
     
@@ -579,7 +645,7 @@ static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
  */
 int afs_linux_flush(struct file *fp)
 {
-    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
+    struct vcache *vcp = (struct vcache *)FILE_INODE(fp);
     int code = 0;
     cred_t *credp;
 
@@ -683,34 +749,21 @@ static int afs_linux_revalidate(struct dentry *dp)
     int code;
     cred_t *credp;
     struct vrequest treq;
-    struct vcache *vcp = ITOAFS(dp->d_inode);
-    struct vcache *rootvp = NULL;
+    struct vcache *vcp = (struct vcache*)dp->d_inode;
 
     AFS_GLOCK();
-
-    if (afs_fakestat_enable && vcp->mvstat == 1 && vcp->mvid &&
-	(vcp->states & CMValid) && (vcp->states & CStatd)) {
-	ObtainSharedLock(&afs_xvcache, 680);
-	rootvp = afs_FindVCache(vcp->mvid, 0, 0, 0, 0);
-	ReleaseSharedLock(&afs_xvcache);
-    }
-
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
 #endif
 
     /* Make this a fast path (no crref), since it's called so often. */
     if (vcp->states & CStatd) {
-	if (*dp->d_name.name != '/' && vcp->mvstat == 2) /* root vnode */
+        if (*dp->d_name.name != '/' && vcp->mvstat == 2) /* root vnode */
 	    check_bad_parent(dp); /* check and correct mvid */
-	if (rootvp)
-	    vcache2fakeinode(rootvp, vcp);
-	else
-	    vcache2inode(vcp);
+	vcache2inode(vcp);
 #ifdef AFS_LINUX24_ENV
 	unlock_kernel();
 #endif
-	if (rootvp) afs_PutVCache(rootvp);
 	AFS_GUNLOCK();
 	return 0;
     }
@@ -749,24 +802,15 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
     struct vcache *lookupvcp = NULL;
     int code, bad_dentry = 1;
     struct sysname_info sysState;
-    struct vcache *vcp = ITOAFS(dp->d_inode);
-    struct vcache *parentvcp = ITOAFS(dp->d_parent->d_inode);
+    struct vcache *vcp = (struct vcache*) dp->d_inode;
+    struct vcache *parentvcp = (struct vcache*) dp->d_parent->d_inode;
 
     AFS_GLOCK();
 
-    sysState.allocked = 0;
-
     /* If it's a negative dentry, then there's nothing to do. */
     if (!vcp || !parentvcp)
         goto done;
 
-    /* If it is the AFS root, then there's no chance it needs 
-       revalidating */
-    if (vcp == afs_globalVp) {
-	bad_dentry = 0;
-	goto done;
-    }
-
     if (code = afs_InitReq(&treq, credp))
         goto done;
 
@@ -823,26 +867,26 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
     int code;
     cred_t *credp;
     struct vrequest treq;
-    struct inode *ip = AFSTOI(dp->d_inode);
-    
+    struct inode *ip = (struct inode *)dp->d_inode;
+
     unsigned long timeout = 3*HZ; /* 3 seconds */
-    
-    if (!ip)
+
+if (!ip)
 	printk("negative dentry: %s\n", dp->d_name.name);
- 
+
     if (!(flags & LOOKUP_CONTINUE)) {
 	long diff = CURRENT_TIME - dp->d_parent->d_inode->i_mtime;
-	
+
 	if (diff < 15*60)
 	    timeout = 0;
     }
-    
+
     if (time_after(jiffies, dp->d_time + timeout))
 	goto out_bad;
-    
+
  out_valid:
     return 1;
-    
+
  out_bad:
     return 0;
 }
@@ -851,41 +895,13 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
 /* afs_dentry_iput */
 static void afs_dentry_iput(struct dentry *dp, struct inode *ip)
 {
-    int isglock;
-    if (ICL_SETACTIVE(afs_iclSetp)) {
-	if (!isglock) AFS_GLOCK();
-	afs_Trace3(afs_iclSetp, CM_TRACE_DENTRYIPUT,
-		   ICL_TYPE_POINTER, ip,
-		   ICL_TYPE_STRING, dp->d_parent->d_name.name,
-		   ICL_TYPE_STRING, dp->d_name.name);
-	if (!isglock) AFS_GUNLOCK();
-    }
-
     osi_iput(ip);
 }
 
-static int afs_dentry_delete(struct dentry *dp)
-{
-    int isglock;
-    if (ICL_SETACTIVE(afs_iclSetp)) {
-	if (!isglock) AFS_GLOCK();
-	afs_Trace3(afs_iclSetp, CM_TRACE_DENTRYDELETE, ICL_TYPE_POINTER, 
-		   dp->d_inode, ICL_TYPE_STRING, dp->d_parent->d_name.name,
-		   ICL_TYPE_STRING, dp->d_name.name);
-	if (!isglock) AFS_GUNLOCK();
-    }
-
-    if (dp->d_inode && (ITOAFS(dp->d_inode)->states & CUnlinked))
-	return 1;               /* bad inode? */
-
-    return 0;
-}
-
 #if defined(AFS_LINUX24_ENV)
 struct dentry_operations afs_dentry_operations = {
        d_revalidate:   afs_linux_dentry_revalidate,
        d_iput:         afs_dentry_iput,
-       d_delete:       afs_dentry_delete,
 };
 struct dentry_operations *afs_dops = &afs_dentry_operations;
 #else
@@ -893,7 +909,7 @@ struct dentry_operations afs_dentry_operations = {
 	afs_linux_dentry_revalidate,	/* d_validate(struct dentry *) */
 	NULL,			/* d_hash */
 	NULL,			/* d_compare */
-	afs_dentry_delete,	/* d_delete(struct dentry *) */
+	NULL,			/* d_delete(struct dentry *) */
 	NULL,			/* d_release(struct dentry *) */
 	afs_dentry_iput		/* d_iput(struct dentry *, struct inode *) */
 };
@@ -925,7 +941,7 @@ int afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
     vattr.va_mode = mode;
 
     AFS_GLOCK();
-    code = afs_create(ITOAFS(dip), name, &vattr, NONEXCL, mode,
+    code = afs_create((struct vcache*)dip, name, &vattr, NONEXCL, mode,
 		      (struct vcache**)&ip, credp);
 
     if (!code) {
@@ -974,10 +990,10 @@ int afs_linux_lookup(struct inode *dip, struct dentry *dp)
     struct vcache *vcp=NULL;
     const char *comp = dp->d_name.name;
     AFS_GLOCK();
-    code = afs_lookup(ITOAFS(dip), comp, &vcp, credp);
+    code = afs_lookup((struct vcache *)dip, comp, &vcp, credp);
 
     if (vcp) {
-	struct inode *ip = AFSTOI(vcp);
+	struct inode *ip = (struct inode*)vcp;
 	/* Reset ops if symlink or directory. */
 #if defined(AFS_LINUX24_ENV)
        if (S_ISREG(ip->i_mode)) {
@@ -999,10 +1015,10 @@ int afs_linux_lookup(struct inode *dip, struct dentry *dp)
 	else if (S_ISLNK(ip->i_mode))
 	    ip->i_op = &afs_symlink_iops;
 #endif
-    } 
     dp->d_time = jiffies;
     dp->d_op = afs_dops;
-    d_add(dp, AFSTOI(vcp));
+    d_add(dp, (struct inode*)vcp);
+    } 
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -1036,7 +1052,7 @@ int afs_linux_link(struct dentry *olddp, struct inode *dip,
     d_drop(newdp);
 
     AFS_GLOCK();
-    code = afs_link(ITOAFS(oldip), ITOAFS(dip), name, credp);
+    code = afs_link((struct vcache*)oldip, (struct vcache*)dip, name, credp);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -1048,9 +1064,10 @@ int afs_linux_unlink(struct inode *dip, struct dentry *dp)
     int code;
     cred_t *credp = crref();
     const char *name = dp->d_name.name;
+    int putback = 0;
 
     AFS_GLOCK();
-    code = afs_remove(ITOAFS(dip), name, credp);
+    code = afs_remove((struct vcache*)dip, name, credp);
     AFS_GUNLOCK();
     if (!code)
 	d_drop(dp);
@@ -1074,7 +1091,7 @@ int afs_linux_symlink(struct inode *dip, struct dentry *dp,
 
     AFS_GLOCK();
     VATTR_NULL(&vattr);
-    code = afs_symlink(ITOAFS(dip), name, &vattr, target, credp);
+    code = afs_symlink((struct vcache*)dip, name, &vattr, target, credp);
     AFS_GUNLOCK();
     crfree(credp);
     return -code;
@@ -1092,7 +1109,7 @@ int afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
     VATTR_NULL(&vattr);
     vattr.va_mask = ATTR_MODE;
     vattr.va_mode = mode;
-    code = afs_mkdir(ITOAFS(dip), name, &vattr, &tvcp, credp);
+    code = afs_mkdir((struct vcache*)dip, name, &vattr, &tvcp, credp);
 
     if (tvcp) {
 	tvcp->v.v_op = &afs_dir_iops;
@@ -1101,7 +1118,7 @@ int afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
 #endif
 	dp->d_op = afs_dops;
 	dp->d_time = jiffies;
-	d_instantiate(dp, AFSTOI(tvcp));
+	d_instantiate(dp, (struct inode*)tvcp);
     }
 
     AFS_GUNLOCK();
@@ -1116,7 +1133,7 @@ int afs_linux_rmdir(struct inode *dip, struct dentry *dp)
     const char *name = dp->d_name.name;
 
     AFS_GLOCK();
-    code = afs_rmdir(ITOAFS(dip), name, credp);
+    code = afs_rmdir((struct vcache*)dip, name, credp);
 
     /* Linux likes to see ENOTEMPTY returned from an rmdir() syscall
      * that failed because a directory is not empty. So, we map
@@ -1157,12 +1174,12 @@ int afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 	d_drop(newdp);
     }
     AFS_GLOCK();
-    code = afs_rename(ITOAFS(oldip), oldname, ITOAFS(newip),
+    code = afs_rename((struct vcache*)oldip, oldname, (struct vcache*)newip,
 		      newname, credp);
     AFS_GUNLOCK();
 
     if (!code) {
-        /* update time so it doesn't expire immediately */
+	/* update time so it doesn't expire immediately */
 	newdp->d_time = jiffies;
 	d_move(olddp, newdp);
     }
@@ -1184,8 +1201,8 @@ static int afs_linux_ireadlink(struct inode *ip, char *target, int maxlen,
     uio_t tuio;
     struct iovec iov;
 
-    setup_uio(&tuio, &iov, target, 0, maxlen, UIO_READ, seg);
-    code = afs_readlink(ITOAFS(ip), &tuio, credp);
+    setup_uio(&tuio, &iov, target, (afs_offs_t) 0, maxlen, UIO_READ, seg);
+    code = afs_readlink((struct vcache*)ip, &tuio, credp);
     crfree(credp);
 
     if (!code)
@@ -1221,6 +1238,7 @@ struct dentry * afs_linux_follow_link(struct dentry *dp,
     char *name;
     struct dentry *res;
 
+
     AFS_GLOCK();
     name = osi_Alloc(PATH_MAX+1);
     if (!name) {
@@ -1257,10 +1275,10 @@ int afs_linux_readpage(struct file *fp, struct page *pp)
     cred_t *credp = crref();
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
     char *address;
-    loff_t offset = pp->index << PAGE_CACHE_SHIFT;
+    afs_offs_t offset = pp->index << PAGE_CACHE_SHIFT;
 #else
     ulong address = afs_linux_page_address(pp);
-    loff_t offset = pageoff(pp);
+    afs_offs_t offset = pageoff(pp);
 #endif
     uio_t tuio;
     struct iovec iovec;
@@ -1273,7 +1291,6 @@ int afs_linux_readpage(struct file *fp, struct page *pp)
 	       ICL_TYPE_POINTER, pp,
 	       ICL_TYPE_INT32, cnt,
 	       ICL_TYPE_INT32, 99999); /* not a possible code value */
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
     address = kmap(pp);
     ClearPageError(pp);
@@ -1287,7 +1304,7 @@ int afs_linux_readpage(struct file *fp, struct page *pp)
 
     setup_uio(&tuio, &iovec, (char*)address, offset, PAGESIZE,
 	      UIO_READ, AFS_UIOSYS);
-    code = afs_rdwr(ITOAFS(ip), &tuio, UIO_READ, 0, credp);
+    code = afs_rdwr((struct vcache*)ip, &tuio, UIO_READ, 0, credp);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
     unlock_kernel();
 #endif
@@ -1297,10 +1314,10 @@ int afs_linux_readpage(struct file *fp, struct page *pp)
 	    memset((void*)(address+(PAGESIZE-tuio.uio_resid)), 0,
 		   tuio.uio_resid);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
-	flush_dcache_page(pp);
-	SetPageUptodate(pp);
+        flush_dcache_page(pp);
+        SetPageUptodate(pp);
 #else
-	set_bit(PG_uptodate, &pp->flags);
+        set_bit(PG_uptodate, &pp->flags);
 #endif
     }
 
@@ -1331,12 +1348,6 @@ int afs_linux_writepage(struct page *pp)
     unsigned long end_index;
     unsigned offset = PAGE_CACHE_SIZE;
     long status;
- 
-#ifdef PageLaunder
-    if (PageLaunder(pp)) {
-        return(fail_writepage(pp));
-    }
-#endif
 
     inode = (struct inode *) mapping->host;
     end_index = inode->i_size >> PAGE_CACHE_SHIFT;
@@ -1388,7 +1399,7 @@ int afs_linux_permission(struct inode *ip, int mode)
     if (mode & MAY_EXEC) tmp |= VEXEC;
     if (mode & MAY_READ) tmp |= VREAD;
     if (mode & MAY_WRITE) tmp |= VWRITE;
-    code = afs_access(ITOAFS(ip), tmp, credp);
+    code = afs_access((struct vcache*)ip, tmp, credp);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -1406,9 +1417,9 @@ int afs_linux_writepage_sync(struct inode *ip, struct page *pp,
                         unsigned long offset,
                         unsigned int count)
 {
-    struct vcache *vcp = ITOAFS(ip);
+    struct vcache *vcp = (struct vcache *) ip;
     char *buffer;
-    loff_t base;
+    afs_offs_t base;
     int code = 0;
     cred_t *credp;
     uio_t tuio;
@@ -1423,24 +1434,13 @@ int afs_linux_writepage_sync(struct inode *ip, struct page *pp,
               ICL_TYPE_POINTER, pp,
               ICL_TYPE_INT32, atomic_read(&pp->count),
               ICL_TYPE_INT32, 99999);
-
     setup_uio(&tuio, &iovec, buffer, base, count, UIO_WRITE, AFS_UIOSYS);
 
     code = afs_write(vcp, &tuio, f_flags, credp, 0);
 
     vcache2inode(vcp);
 
-    if (!code && afs_stats_cmperf.cacheCurrDirtyChunks >
-		 afs_stats_cmperf.cacheMaxDirtyChunks) {
-	struct vrequest treq;
-
-	ObtainWriteLock(&vcp->lock, 533);
-	if (!afs_InitReq(&treq, credp))
-	    code = afs_DoPartialWrite(vcp, &treq);
-	ReleaseWriteLock(&vcp->lock);
-    }
     code = code ? -code : count - tuio.uio_resid;
-
     afs_Trace4(afs_iclSetp, CM_TRACE_UPDATEPAGE, ICL_TYPE_POINTER, vcp,
               ICL_TYPE_POINTER, pp,
               ICL_TYPE_INT32, atomic_read(&pp->count),
@@ -1450,7 +1450,7 @@ int afs_linux_writepage_sync(struct inode *ip, struct page *pp,
     kunmap(pp);
 
     return code;
-}
+} 
 
 static int
 afs_linux_updatepage(struct file *file, struct page *page, 
@@ -1470,7 +1470,7 @@ int afs_linux_updatepage(struct file *fp, struct page *pp,
 			 unsigned long offset,
 			 unsigned int count, int sync)
 {
-    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
+    struct vcache *vcp = (struct vcache *)FILE_INODE(fp);
     u8 *page_addr = (u8*) afs_linux_page_address(pp);
     int code = 0;
     cred_t *credp;
@@ -1485,8 +1485,8 @@ int afs_linux_updatepage(struct file *fp, struct page *pp,
 	       ICL_TYPE_POINTER, pp,
 	       ICL_TYPE_INT32, atomic_read(&pp->count),
 	       ICL_TYPE_INT32, 99999);
-    setup_uio(&tuio, &iovec, page_addr + offset, pageoff(pp) + offset, count,
-	      UIO_WRITE, AFS_UIOSYS);
+    setup_uio(&tuio, &iovec, page_addr + offset, (afs_offs_t)(pageoff(pp) + offset),
+		count, UIO_WRITE, AFS_UIOSYS);
 
     code = afs_write(vcp, &tuio, fp->f_flags, credp, 0);
 
