@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.14.2.18 2003/03/23 19:47:53 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.25 2002/02/16 18:23:49 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -21,7 +21,7 @@ RCSID("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.14.2.18 2003/03/23 19:47:53
 #ifdef AFS_SGI62_ENV
 #include "../h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV)
 #include "netinet/in_var.h"
 #endif
 #endif /* !defined(UKERNEL) */
@@ -83,7 +83,6 @@ static int afs_CacheInit_Done = 0;
 static int afs_Go_Done = 0;
 extern struct interfaceAddr afs_cb_interface;
 static int afs_RX_Running = 0;
-static int afs_InitSetup_done = 0;
 
 static int
 Afscall_icl(long opcode, long p1, long p2, long p3, long p4, long *retval);
@@ -100,9 +99,6 @@ static int afs_InitSetup(int preallocs)
     extern void afs_InitStats();
     int code;
 
-    if (afs_InitSetup_done)
-	return;
-
 #ifndef AFS_NOSTATS
     /*
      * Set up all the AFS statistics variables.  This should be done
@@ -125,188 +121,13 @@ static int afs_InitSetup(int preallocs)
     /* resource init creates the services */
     afs_ResourceInit(preallocs);
 
-    afs_InitSetup_done = 1;
-    afs_osi_Wakeup(&afs_InitSetup_done);
-
     return code;
 }
-#if defined(AFS_LINUX24_ENV) && defined(COMPLETION_H_EXISTS)
-struct afsd_thread_info {
-    unsigned long parm;
-    struct completion *complete;
-};
-
-static int afsd_thread(void *rock) {
-     struct afsd_thread_info  *arg=rock;
-     unsigned long parm=arg->parm;
-#ifdef SYS_SETPRIORITY_EXPORTED
-     int (*sys_setpriority)(int,int,int) = sys_call_table[__NR_setpriority];
-#endif
-     daemonize(); /* doesn't do much, since we were forked from keventd, but
-                     does call mm_release, which wakes up our parent (since it
-                     used CLONE_VFORK) */
-     reparent_to_init();
-     afs_osi_MaskSignals();
-     switch (parm) {
-     case AFSOP_START_RXCALLBACK:
-          sprintf(current->comm, "afs_cbstart");
-          AFS_GLOCK();
-          complete(arg->complete);
-          afs_CB_Running = 1;     
-          while (afs_RX_Running != 2)
-               afs_osi_Sleep(&afs_RX_Running);
-          sprintf(current->comm, "afs_callback");
-          afs_RXCallBackServer();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     case AFSOP_START_AFS:
-          sprintf(current->comm, "afs_afsstart");
-          AFS_GLOCK();
-          complete(arg->complete);
-          AFS_Running = 1;
-          while (afs_initState < AFSOP_START_AFS) 
-               afs_osi_Sleep(&afs_initState);
-          DInit(AFS_MINBUFFERS);          
-          afs_initState = AFSOP_START_BKG;
-          afs_osi_Wakeup(&afs_initState);
-          sprintf(current->comm, "afsd");
-          afs_Daemon();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     case AFSOP_START_BKG:
-          sprintf(current->comm, "afs_bkgstart");
-          AFS_GLOCK();
-          complete(arg->complete);
-          while (afs_initState < AFSOP_START_BKG) 
-               afs_osi_Sleep(&afs_initState);
-          if (afs_initState < AFSOP_GO) {
-               afs_initState = AFSOP_GO;
-               afs_osi_Wakeup(&afs_initState);
-          }
-          sprintf(current->comm, "afs_background");          
-          afs_BackgroundDaemon();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     case AFSOP_START_TRUNCDAEMON:
-          sprintf(current->comm, "afs_trimstart");
-          AFS_GLOCK();
-          complete(arg->complete);
-          while (afs_initState < AFSOP_GO) 
-               afs_osi_Sleep(&afs_initState);
-          sprintf(current->comm, "afs_cachetrim");
-          afs_CacheTruncateDaemon();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     case AFSOP_START_CS:
-          sprintf(current->comm, "afs_checkserver");
-          AFS_GLOCK();
-          complete(arg->complete);
-          afs_CheckServerDaemon();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     case AFSOP_RXEVENT_DAEMON:
-          sprintf(current->comm, "afs_evtstart");
-#ifdef SYS_SETPRIORITY_EXPORTED
-        sys_setpriority(PRIO_PROCESS,0,-10);
-#else
-#ifdef CURRENT_INCLUDES_NICE
-        current->nice=-10;
-#endif
-#endif
-          AFS_GLOCK();
-          complete(arg->complete);
-          while (afs_initState < AFSOP_START_BKG) 
-               afs_osi_Sleep(&afs_initState);
-          sprintf(current->comm, "afs_rxevent");
-          afs_rxevent_daemon();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     case AFSOP_RXLISTENER_DAEMON:
-          sprintf(current->comm, "afs_lsnstart");
-#ifdef SYS_SETPRIORITY_EXPORTED
-        sys_setpriority(PRIO_PROCESS,0,-10);
-#else
-#ifdef CURRENT_INCLUDES_NICE
-        current->nice=-10;
-#endif
-#endif
-          AFS_GLOCK();
-          complete(arg->complete);
-          afs_initState = AFSOP_START_AFS;
-          afs_osi_Wakeup(&afs_initState);
-          afs_RX_Running = 2;
-          afs_osi_Wakeup(&afs_RX_Running);
-          afs_osi_RxkRegister();
-          sprintf(current->comm, "afs_rxlistener");
-          rxk_Listener();
-          AFS_GUNLOCK();
-          complete_and_exit(0,0);
-          break;
-     default:
-          printf("Unknown op %d in StartDaemon()\n");
-          break;
-     }
-     return 0;
-}
-
-void afsd_launcher(void *rock) {
-     if (!kernel_thread(afsd_thread,rock, CLONE_VFORK|SIGCHLD))
-	printf("kernel_thread failed. afs startup will not complete\n");
-}
-
-void afs_DaemonOp(long parm, long parm2, long parm3, long parm4, long parm5, 
-             long parm6) 
-{
-     int code;
-     DECLARE_COMPLETION(c);
-     struct tq_struct tq;
-     struct afsd_thread_info info;     
-     if (parm == AFSOP_START_RXCALLBACK) {
-          if (afs_CB_Running) return;
-     } else if (parm == AFSOP_RXLISTENER_DAEMON) {
-          if (afs_RX_Running) return;
-          afs_RX_Running=1;
-          code = afs_InitSetup(parm2);
-          if (parm3) {
-               rx_enablePeerRPCStats();
-          }
-          if (parm4) {
-               rx_enableProcessRPCStats();
-          }
-          if (code)
-               return;
-     } else if (parm == AFSOP_START_AFS) {
-          if (AFS_Running) return;
-     } /* other functions don't need setup in the parent */     
-     info.complete=&c;
-     info.parm=parm;
-     tq.sync=0;
-     INIT_LIST_HEAD(&tq.list);
-     tq.routine=afsd_launcher;
-     tq.data=&info;
-     schedule_task(&tq);
-     AFS_GUNLOCK();
-     /* we need to wait cause we passed stack pointers around.... */
-     wait_for_completion(&c);
-     AFS_GLOCK();
-}
-#endif
 
 afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 long parm, parm2, parm3, parm4, parm5, parm6;
 {
     afs_int32 code = 0;
-#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
-    size_t bufferSize;	
-#else /* AFS_SGI61_ENV */
-    u_int bufferSize;	
-#endif /* AFS_SGI61_ENV */
 
     AFS_STATCNT(afs_syscall_call);
 #ifdef	AFS_SUN5_ENV
@@ -331,12 +152,6 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 #endif
     }
     AFS_GLOCK();
-#if defined(AFS_LINUX24_ENV) && defined(COMPLETION_H_EXISTS) && !defined(UKERNEL)
-    if (parm < AFSOP_ADDCELL || parm == AFSOP_RXEVENT_DAEMON
-         || parm == AFSOP_RXLISTENER_DAEMON) {
-         afs_DaemonOp(parm,parm2,parm3,parm4,parm5,parm6);
-    }
-#else
     if (parm == AFSOP_START_RXCALLBACK) {
 	if (afs_CB_Running) goto out;
 	afs_CB_Running = 1;
@@ -390,11 +205,21 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 #endif
     else if (parm == AFSOP_START_AFS) {
 	/* afs daemon */
+	afs_int32 temp;
+
 	if (AFS_Running) goto out;
 	AFS_Running = 1;
 	while (afs_initState < AFSOP_START_AFS) 
 	    afs_osi_Sleep(&afs_initState);
 
+#if defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+	temp = AFS_MINBUFFERS;  /* Should fix this soon */
+#else
+	temp = ((afs_bufferpages * NBPG)>>11);	/* number of 2k buffers we could get from all of the buffer space */
+	temp = temp>>2;	/* don't take more than 25% (our magic parameter) */
+	if (temp < AFS_MINBUFFERS) temp = AFS_MINBUFFERS;   /* although we really should have this many */
+#endif
+	DInit(temp);
 	afs_initState = AFSOP_START_BKG;
 	afs_osi_Wakeup(&afs_initState);
 	afs_osi_Invisible();
@@ -454,32 +279,13 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 #endif
     }
 #endif	
-#endif
-    else if (parm == AFSOP_BASIC_INIT) {
-	afs_int32 temp;
-
-	while (!afs_InitSetup_done)
-	    afs_osi_Sleep(&afs_InitSetup_done);
-
-#if defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
-	temp = AFS_MINBUFFERS;  /* Should fix this soon */
-#else
-	/* number of 2k buffers we could get from all of the buffer space */
-	temp = ((afs_bufferpages * NBPG)>>11);
-	temp = temp>>2;	/* don't take more than 25% (our magic parameter) */
-	if (temp < AFS_MINBUFFERS)
-	    temp = AFS_MINBUFFERS; /* though we really should have this many */
-#endif
-	DInit(temp);
-	afs_rootFid.Fid.Volume = 0;
-	code = 0;
-    }
     else if (parm == AFSOP_ADDCELL) {
 	/* add a cell.  Parameter 2 is 8 hosts (in net order),  parm 3 is the null-terminated
 	 name.  Parameter 4 is the length of the name, including the null.  Parm 5 is the
 	 home cell flag (0x1 bit) and the nosuid flag (0x2 bit) */
 	struct afsop_cell tcell;
 
+	while (afs_initState < AFSOP_START_BKG) afs_osi_Sleep(&afs_initState);
 	AFS_COPYIN((char *)parm2, (char *)tcell.hosts, sizeof(tcell.hosts), code);
 	if (!code) {
 	    if (parm4 > sizeof(tcell.cellName)) 
@@ -488,19 +294,22 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 		AFS_COPYIN((char *)parm3, tcell.cellName, parm4, code);
 		if (!code) 
 		    afs_NewCell(tcell.cellName, tcell.hosts, parm5,
-			        NULL, 0, 0, 0);
+				(char *)0, (u_short)0, (u_short)0, (int)0);
 	    }
 	}
     } else if (parm == AFSOP_ADDCELL2) {
 	struct afsop_cell tcell;
 	char *tbuffer = osi_AllocSmallSpace(AFS_SMALLOCSIZ), *lcnamep = 0;
 	char *tbuffer1 = osi_AllocSmallSpace(AFS_SMALLOCSIZ), *cnamep = 0;
+#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+	size_t bufferSize;	
+#else /* AFS_SGI61_ENV */
+	u_int bufferSize;	
+#endif /* AFS_SGI61_ENV */
 	int cflags = parm4;
 
-#if 0
-	/* wait for basic init - XXX can't find any reason we need this? */
+	/* wait for basic init */
 	while (afs_initState < AFSOP_START_BKG) afs_osi_Sleep(&afs_initState);
-#endif
 
 	AFS_COPYIN((char *)parm2, (char *)tcell.hosts, sizeof(tcell.hosts), code);
 	if (!code) {
@@ -515,7 +324,7 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 		}
 		if (!code)
 		    afs_NewCell(tbuffer1, tcell.hosts, cflags, 
-				lcnamep, 0, 0, 0);
+				lcnamep, (u_short)0, (u_short)0, (int)0);
 	    }
 	}
 	osi_FreeSmallSpace(tbuffer);
@@ -527,33 +336,34 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	 * parm2 is the alias name
 	 * parm3 is the real cell name
 	 */
+#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+	size_t bufferSize;
+#else /* AFS_SGI61_ENV */
+	u_int bufferSize;	
+#endif /* AFS_SGI61_ENV */
 	char *aliasName = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
 	char *cellName = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
 
 	AFS_COPYINSTR((char *)parm2, aliasName, AFS_SMALLOCSIZ, &bufferSize, code);
 	if (!code) AFS_COPYINSTR((char *)parm3, cellName, AFS_SMALLOCSIZ, &bufferSize, code);
-	if (!code) afs_NewCellAlias(aliasName, cellName);
+	if (!code) afs_NewCell(aliasName,	/* new entry name */
+			       0,		/* host list */
+			       CAlias,		/* flags */
+			       (char *) 0,	/* linked cell */
+			       0, 0,		/* fs & vl ports */
+			       0,		/* timeout */
+			       cellName);	/* real cell name */
 
 	osi_FreeSmallSpace(aliasName);
 	osi_FreeSmallSpace(cellName);
     }
-    else if (parm == AFSOP_SET_THISCELL) {
-	/*
-	 * Call arguments:
-	 * parm2 is the primary cell name
-	 */
-	char *cell = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
-
-	AFS_COPYINSTR((char *) parm2, cell, AFS_SMALLOCSIZ, &bufferSize, code);
-	if (!code)
-	    afs_SetPrimaryCell(cell);
-	osi_FreeSmallSpace(cell);
-    }
     else if (parm == AFSOP_CACHEINIT) {
 	struct afs_cacheParams cparms;
 
 	if (afs_CacheInit_Done) goto out;
 
+	/* wait for basic init */
+	while (afs_initState < AFSOP_START_BKG) afs_osi_Sleep(&afs_initState);
 	AFS_COPYIN((char *)parm2, (caddr_t) &cparms, sizeof(cparms), code);
 	if (code) {
 #if	defined(AFS_SUN5_ENV) || defined(AFS_OSF_ENV) || defined (AFS_SGI64_ENV) || defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
@@ -602,6 +412,12 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	code = afs_InitCacheFile((char *) 0, ainode);
     }
     else if (parm == AFSOP_ROOTVOLUME) {
+#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+	size_t bufferSize;
+#else /* AFS_SGI61_ENV */
+	u_int bufferSize;
+#endif /* AFS_SGI61_ENV */
+
 	/* wait for basic init */
 	while (afs_initState < AFSOP_START_BKG) afs_osi_Sleep(&afs_initState);
 
@@ -611,31 +427,29 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	}
 	else code = 0;
     }
-    else if (parm == AFSOP_CACHEFILE ||
-	     parm == AFSOP_CACHEINFO ||
-	     parm == AFSOP_VOLUMEINFO ||
-	     parm == AFSOP_AFSLOG ||
-	     parm == AFSOP_CELLINFO) {
+    else if (parm == AFSOP_CACHEFILE || parm == AFSOP_CACHEINFO ||
+	      parm == AFSOP_VOLUMEINFO || parm == AFSOP_AFSLOG) {
 	char *tbuffer = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
+#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+	size_t bufferSize;
+#else /* AFS_SGI61_ENV */
+	u_int bufferSize;
+#endif /* AFS_SGI61_ENV */
 
+	/* wait for basic init */
+	while (afs_initState < AFSOP_START_BKG) afs_osi_Sleep(&afs_initState);
 	code = 0;
-	AFS_COPYINSTR((char *) parm2, tbuffer, AFS_SMALLOCSIZ,
-		      &bufferSize, code);
+	AFS_COPYINSTR((char *)parm2, tbuffer, AFS_SMALLOCSIZ, &bufferSize, code);
 	if (code) {
 	    osi_FreeSmallSpace(tbuffer);
 	    goto out;
 	}
 	if (!code) {
-	    tbuffer[AFS_SMALLOCSIZ-1] = '\0';	/* null-terminate the name */
-	    /* We have the cache dir copied in.  Call the cache init routine */
-	    if (parm == AFSOP_CACHEFILE)
-		code = afs_InitCacheFile(tbuffer, 0);
-	    else if (parm == AFSOP_CACHEINFO)
-		code = afs_InitCacheInfo(tbuffer);
-	    else if (parm == AFSOP_VOLUMEINFO)
-		code = afs_InitVolumeInfo(tbuffer);
-	    else if (parm == AFSOP_CELLINFO)
-		code = afs_InitCellInfo(tbuffer);
+	    tbuffer[AFS_SMALLOCSIZ-1] = 0;	/* null-terminate the name */
+	    /* we now have the cache dir copied in.  Call the cache init routines */
+	    if (parm == AFSOP_CACHEFILE) code = afs_InitCacheFile(tbuffer, 0);
+	    else if (parm == AFSOP_CACHEINFO) code = afs_InitCacheInfo(tbuffer);
+	    else if (parm == AFSOP_VOLUMEINFO) code = afs_InitVolumeInfo(tbuffer);
 	}
 	osi_FreeSmallSpace(tbuffer);
     }
@@ -731,9 +545,10 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 #endif
 	afs_cold_shutdown = 0;
 	if (parm == 1) afs_cold_shutdown = 1;
-	if (afs_globalVFS != 0) 
-	    afs_warn("AFS isn't unmounted yet!\n");
-
+	if (afs_globalVFS != 0) {
+	    afs_warn("AFS isn't unmounted yet! Call aborted\n");
+	    code = EACCES;
+	}
 	afs_shutdown();
     }
 
@@ -805,19 +620,11 @@ long parm, parm2, parm3, parm4, parm5, parm6;
       }
 #else /* AFS_USERSPACE_IP_ADDR */
       struct ifnet *tifnp;
-#ifdef AFS_DARWIN60_ENV
-      struct ifaddr *tifadp = (struct ifaddr *) 0;
-#else
       struct in_ifaddr *tifadp = (struct in_ifaddr *) 0;
-#endif
       extern struct ifnet *rxi_FindIfnet();
       tifnp = rxi_FindIfnet(parm2, &tifadp);  /* make iterative */
       if (tifnp && tifadp) {
-#ifdef AFS_DARWIN60_ENV
-	 mask = ((struct sockaddr_in *)tifadp->ifa_netmask)->sin_addr.s_addr;
-#else
 	 mask = tifadp->ia_subnetmask;
-#endif
       } else {
 	 code = -1;
       }
@@ -840,7 +647,7 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	AFS_COPYIN((afs_int32 *)parm2, cellname, cellLen, code);
 	AFS_COPYIN((afs_int32 *)parm3, kmsg, kmsgLen, code);
 	if (!code) {
-	    code = afs_AFSDBHandler(cellname, cellLen, kmsg);
+	    code = afs_AfsdbHandler(cellname, cellLen, kmsg);
 	    if (*cellname == 1) *cellname = 0;
 	    if (code == -2) {	/* Shutting down? */
 		*cellname = 1;
@@ -855,10 +662,6 @@ long parm, parm2, parm3, parm4, parm5, parm6;
     else if (parm == AFSOP_SET_DYNROOT) {
 	code = afs_SetDynrootEnable(parm2);
     }
-    else if (parm == AFSOP_SET_FAKESTAT) {
-    	afs_fakestat_enable = parm2;
-    	code = 0;
-    }
     else
       code = EINVAL;
 
@@ -1178,9 +981,6 @@ asmlinkage int afs_syscall(long syscall, long parm1, long parm2, long parm3,
     long linux_ret=0;
     long *retval = &linux_ret;
     long eparm[4]; /* matches AFSCALL_ICL in fstrace.c */
-#ifdef AFS_SPARC64_LINUX24_ENV
-    afs_int32 eparm32[4];
-#endif
     /* eparm is also used by AFSCALL_CALL in afsd.c */
 #else
 #if defined(UKERNEL)
@@ -1243,25 +1043,6 @@ Afs_syscall ()
     uap->parm2 = parm2;
     uap->parm3 = parm3;
     if (syscall == AFSCALL_ICL || syscall == AFSCALL_CALL) {
-#ifdef AFS_SPARC64_LINUX24_ENV
-/* from arch/sparc64/kernel/sys_sparc32.c */
-#define AA(__x)                                \
-({     unsigned long __ret;            \
-       __asm__ ("srl   %0, 0, %0"      \
-                : "=r" (__ret)         \
-                : "0" (__x));          \
-       __ret;                          \
-})
-
-
-	if (current->thread.flags & SPARC_FLAG_32BIT) {
-	AFS_COPYIN((char*)parm4, (char*)eparm32, sizeof(eparm32), code);
-	eparm[0]=AA(eparm32[0]);
-	eparm[1]=AA(eparm32[1]);
-	eparm[2]=AA(eparm32[2]);
-#undef AA
-} else
-#endif
 	AFS_COPYIN((char*)parm4, (char*)eparm, sizeof(eparm), code);
 	uap->parm4 = eparm[0];
 	uap->parm5 = eparm[1];
@@ -1464,7 +1245,7 @@ afs_shutdown()
     }
 #ifdef AFS_AFSDB_ENV
     afs_warn("AFSDB... ");
-    afs_StopAFSDB();
+    afs_StopAfsdb();
     while (afs_termState == AFSOP_STOP_AFSDB)
 	afs_osi_Sleep(&afs_termState);
 #endif
@@ -1965,7 +1746,7 @@ afs_icl_UseAddr(type)
   int type;
 {
     if (type == ICL_TYPE_HYPER || type == ICL_TYPE_STRING
-	|| type == ICL_TYPE_FID)
+	|| type == ICL_TYPE_FID || type == ICL_TYPE_INT64)
 	return 1;
     else
 	return 0;
@@ -2076,6 +1857,25 @@ afs_icl_AppendRecord(logp, op, types, p1, p2, p3, p4)
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p1)->high);
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p1)->low);
 	}
+	else if (t1 == ICL_TYPE_INT64) {
+#ifdef AFSLITTLE_ENDIAN
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p1)[1]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p1)[0]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) p1);
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+#endif /* AFS_64BIT_CLIENT */
+#else /* AFSLITTLE_ENDIAN */
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p1)[0]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p1)[1]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+	    ICL_APPENDINT32(logp, (afs_int32) p1);
+#endif /* AFS_64BIT_CLIENT */
+#endif /* AFSLITTLE_ENDIAN */
+        }
 	else if (t1 == ICL_TYPE_FID) {
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p1)[0]);
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p1)[1]);
@@ -2095,6 +1895,25 @@ afs_icl_AppendRecord(logp, op, types, p1, p2, p3, p4)
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p2)->high);
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p2)->low);
 	}
+	else if (t2 == ICL_TYPE_INT64) {
+#ifdef AFSLITTLE_ENDIAN
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p2)[1]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p2)[0]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) p2);
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+#endif /* AFS_64BIT_CLIENT */
+#else /* AFSLITTLE_ENDIAN */
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p2)[0]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p2)[1]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+	    ICL_APPENDINT32(logp, (afs_int32) p2);
+#endif /* AFS_64BIT_CLIENT */
+#endif /* AFSLITTLE_ENDIAN */
+        }
 	else if (t2 == ICL_TYPE_FID) {
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p2)[0]);
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p2)[1]);
@@ -2114,6 +1933,25 @@ afs_icl_AppendRecord(logp, op, types, p1, p2, p3, p4)
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p3)->high);
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p3)->low);
 	}
+	else if (t3 == ICL_TYPE_INT64) {
+#ifdef AFSLITTLE_ENDIAN
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p3)[1]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p3)[0]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) p3);
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+#endif /* AFS_64BIT_CLIENT */
+#else /* AFSLITTLE_ENDIAN */
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p3)[0]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p3)[1]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+	    ICL_APPENDINT32(logp, (afs_int32) p3);
+#endif /* AFS_64BIT_CLIENT */
+#endif /* AFSLITTLE_ENDIAN */
+        }
 	else if (t3 == ICL_TYPE_FID) {
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p3)[0]);
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p3)[1]);
@@ -2133,6 +1971,25 @@ afs_icl_AppendRecord(logp, op, types, p1, p2, p3, p4)
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p4)->high);
 	    ICL_APPENDINT32(logp, (afs_int32)((struct afs_hyper_t *)p4)->low);
 	}
+	else if (t4 == ICL_TYPE_INT64) {
+#ifdef AFSLITTLE_ENDIAN
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p4)[1]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p4)[0]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) p4);
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+#endif /* AFS_64BIT_CLIENT */
+#else /* AFSLITTLE_ENDIAN */
+#ifdef AFS_64BIT_CLIENT
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p4)[0]);
+	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p4)[1]);
+#else /* AFS_64BIT_CLIENT */
+	    ICL_APPENDINT32(logp, (afs_int32) 0);
+	    ICL_APPENDINT32(logp, (afs_int32) p4);
+#endif /* AFS_64BIT_CLIENT */
+#endif /* AFSLITTLE_ENDIAN */
+        }
 	else if (t4 == ICL_TYPE_FID) {
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p4)[0]);
 	    ICL_APPENDINT32(logp, (afs_int32)((afs_int32 *)p4)[1]);
