@@ -18,21 +18,19 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_volume.c,v 1.6.2.7 2003/05/20 04:23:07 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_volume.c,v 1.13 2002/02/05 21:58:03 shadow Exp $");
 
 #include "../afs/stds.h"
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 
 #if !defined(UKERNEL)
-#if !defined(AFS_LINUX20_ENV)
 #include <net/if.h>
-#endif
 #include <netinet/in.h>
 
 #ifdef AFS_SGI62_ENV
 #include "../h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV)
 #include <netinet/in_var.h>
 #endif /* ! AFS_HPUX110_ENV */
 #endif /* !defined(UKERNEL) */
@@ -114,7 +112,7 @@ struct volume *afs_UFSGetVolSlot()
 
 {
     register struct volume *tv, **lv;
-    struct osi_file *tfile;
+    register char *tfile;
     register afs_int32 i, code;
     afs_int32 bestTime;
     struct volume *bestVp, **bestLp;
@@ -319,6 +317,7 @@ void afs_CheckVolumeNames(flags)
 		    ReleaseReadLock(&afs_xvcache);
 
 		    ObtainWriteLock(&afs_xcbhash, 485);
+		    /* LOCKXXX: We aren't holding tvc write lock? */
 		    afs_DequeueCallback(tvc);
 		    tvc->states &= ~CStatd;
 		    ReleaseWriteLock(&afs_xcbhash);
@@ -463,7 +462,7 @@ static struct volume *afs_SetupVolume(volid, aname, ve, tcell, agood, type, areq
     ObtainWriteLock(&afs_xvolume,108);
     i = VHash(volid);
     for (tv = afs_volumes[i]; tv; tv=tv->next) {
-	if (tv->volume == volid && tv->cell == tcell->cellNum) {
+	if (tv->volume == volid && tv->cell == tcell->cell) {
 	    break;
 	}
     }
@@ -472,14 +471,14 @@ static struct volume *afs_SetupVolume(volid, aname, ve, tcell, agood, type, areq
 
 	tv = afs_GetVolSlot();
 	memset((char *)tv, 0, sizeof(struct volume));
-	tv->cell = tcell->cellNum;
+	tv->cell = tcell->cell;
 	RWLOCK_INIT(&tv->lock, "volume lock");
 	tv->next = afs_volumes[i];	/* thread into list */
 	afs_volumes[i] = tv;
 	tv->volume = volid;
 	for (j=fvTable[FVHash(tv->cell,volid)]; j!=0; j=tf->next) {
 	    if (afs_FVIndex != j) {
-		struct osi_file *tfile;
+		char *tfile;
 		tfile = osi_UFSOpen(volumeInode);
 		err = afs_osi_Read(tfile, sizeof(struct fvolume) * j, &staticFVolume, sizeof(struct fvolume));
 		if (err != sizeof(struct fvolume))
@@ -508,12 +507,12 @@ static struct volume *afs_SetupVolume(volid, aname, ve, tcell, agood, type, areq
     ReleaseWriteLock(&afs_xvolume);
     ObtainWriteLock(&tv->lock,111);
     if (type == 2) {
-	InstallUVolumeEntry(tv, uve, tcell->cellNum, tcell, areq);
+	InstallUVolumeEntry(tv, uve, tcell->cell, tcell, areq);
     }
     else if (type == 1)
-	InstallNVolumeEntry(tv, nve, tcell->cellNum);
+	InstallNVolumeEntry(tv, nve, tcell->cell);
     else
-	InstallVolumeEntry(tv, ove, tcell->cellNum);
+	InstallVolumeEntry(tv, ove, tcell->cell);
     if (agood) {
 	if (!tv->name) {
 	    tv->name = afs_osi_Alloc(strlen(aname) + 1);
@@ -575,7 +574,7 @@ static struct volume *afs_NewDynrootVolume(struct VenusFid *fid) {
     tve.volumeId[ROVOL] = fid->Fid.Volume;
     tve.flags = VLF_ROEXISTS;
 
-    tv = afs_SetupVolume(0, bp, (char *) &tve, tcell, 0, 0, 0);
+    tv = afs_SetupVolume(0, bp, &tve, tcell, 0, 0, 0);
     afs_PutCell(tcell, READ_LOCK);
     return tv;
 }
@@ -616,7 +615,7 @@ static struct volume *afs_NewVolumeByName(char *aname, afs_int32 acell, int agoo
     afs_InitReq(&treq, &afs_osi_cred);	/* *must* be unauth for vldb */
     do {
 	tconn = afs_ConnByMHosts(tcell->cellHosts, tcell->vlport,
-				 tcell->cellNum, &treq, SHARED_LOCK);
+				 tcell->cell, &treq, SHARED_LOCK);
 	if (tconn) {
 	    if (tconn->srvr->server->flags & SNO_LHOSTS) {
 		type = 0;
@@ -772,6 +771,15 @@ void InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 
     cellp = afs_GetCell(acell, 0);
 
+    /* This volume, av, is locked. Zero out the serverHosts[] array 
+     * so that if afs_GetServer() decides to replace the server 
+     * struct, we don't deadlock trying to afs_ResetVolumeInfo()
+     * this volume.
+     */
+    for (j=0; j<MAXHOSTS; j++) {
+       av->serverHost[j] = 0;
+    }
+
     /* Step through the VLDB entry making sure each server listed is there */
     for (i=0,j=0; i<ve->nServers; i++) {
 	if ( ((ve->serverFlags[i] & mask) == 0) || (ve->serverFlags[i] & VLSF_DONTUSE) ) {
@@ -839,6 +847,15 @@ void InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
 
     cellp = afs_GetCell(acell, 0);
 
+    /* This volume, av, is locked. Zero out the serverHosts[] array 
+     * so that if afs_GetServer() decides to replace the server 
+     * struct, we don't deadlock trying to afs_ResetVolumeInfo()
+     * this volume.
+     */
+    for (j=0; j<MAXHOSTS; j++) {
+       av->serverHost[j] = 0;
+    }
+
     /* Step through the VLDB entry making sure each server listed is there */
     for (i=0,j=0; i<ve->nServers; i++) {
         if ( ((ve->serverFlags[i] & mask) == 0) || (ve->serverFlags[i] & VLSF_DONTUSE) ) {
@@ -949,7 +966,7 @@ void InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve,
 		memset((char *)&addrs, 0, sizeof(addrs));
 		do {
 		    tconn = afs_ConnByMHosts(tcell->cellHosts, tcell->vlport,
-					     tcell->cellNum, areq, SHARED_LOCK);
+					     tcell->cell, areq, SHARED_LOCK);
 		    if (tconn) {
 #ifdef RX_ENABLE_LOCKS
 			AFS_GUNLOCK();
