@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/kauth/rebuild.c,v 1.7 2001/08/08 00:03:46 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/kauth/rebuild.c,v 1.9 2001/10/05 21:17:18 shadow Exp $");
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -22,6 +22,13 @@ RCSID("$Header: /cvs/openafs/src/kauth/rebuild.c,v 1.7 2001/08/08 00:03:46 shado
 #include <sys/file.h>
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <stdio.h>
 #include <errno.h>
 #include <time.h>
@@ -41,6 +48,8 @@ char *whoami = "kadb_check";
 int fd;
 FILE *out;
 
+void badEntry();
+
 int listuheader, listkheader, listentries, verbose;
 
 int readUbikHeader()
@@ -82,6 +91,7 @@ int readUbikHeader()
   return(0);
 }
 
+void
 PrintHeader(header)
   struct kaheader *header;
 {
@@ -100,6 +110,7 @@ PrintHeader(header)
   printf("stats.cpw commands     = %d\n", header->stats.cpws);
 }
 
+void
 PrintEntry(index, entry)
   afs_int32 index;
   struct kaentry *entry;
@@ -221,6 +232,7 @@ char *EntryName(entryp)
   return(principal);
 }
 
+void
 RebuildEntry(entryp)
   struct kaentry *entryp;
 {
@@ -365,7 +377,6 @@ WorkerBee (as, arock)
   char *arock;
 {
     afs_int32 code;
-    int a;
     char *dbFile;
     char *outFile;
     afs_int32 index;
@@ -556,6 +567,7 @@ WorkerBee (as, arock)
     exit (code != 0);
 }
 
+void
 badEntry(e, i)
   afs_int32 e, i;
 {
@@ -585,7 +597,6 @@ main (argc, argv)
   char *argv[];
 {
   struct cmd_syndesc *ts;
-  struct cmd_item    *ti;
 
   setlinebuf(stdout);
 
