@@ -10,15 +10,20 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/ptserver/readgroup.c,v 1.5 2001/08/08 00:03:54 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/ptserver/readgroup.c,v 1.7 2001/10/05 21:07:17 shadow Exp $");
 
 #include <stdio.h>
-#ifndef AFS_NT40_ENV
-#include <strings.h>
-#else
+#ifdef AFS_NT40_ENV
 #include <WINNT/afsevent.h>
 #endif
 #include <ctype.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/cellconfig.h>
@@ -27,7 +32,9 @@ RCSID("$Header: /cvs/openafs/src/ptserver/readgroup.c,v 1.5 2001/08/08 00:03:54
 #include "pterror.h"
 
 int verbose = 0;
+void skip();
 
+void
 report_error (code, name, gname)
   afs_int32 code;
   char *name;
@@ -66,7 +73,6 @@ char **argv;
     char buf[3000];
     FILE *fp;
     char *ptr;
-    char *aptr;
     char *tmp;
     char *cellname;
     namelist lnames;
@@ -195,6 +201,7 @@ char **argv;
     }
 }
 
+void
 skip(s)
 char **s;
 {
