@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.5 2001/07/12 19:58:54 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.9 2002/02/16 18:23:58 shadow Exp $");
 
 #include <ctype.h>
 #include <errno.h>
@@ -23,6 +23,22 @@ RCSID("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.5 2001/07/12 19:58:54 sha
 #include <netdb.h>
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/rxgen_consts.h>
@@ -464,12 +480,12 @@ TestManyMembers (as, arock)
     population = (char *)malloc (sqr(number)*sizeof(char));
 
     nFilled = 0;
-    bzero (filled, number);
+    memset(filled, 0, number);
     nCleaned = 0;
-    bzero (cleaned, number);
-    bzero (population, sqr(number));
-    bzero (users, number*sizeof(afs_int32));
-    bzero (groups, number*sizeof(afs_int32));
+    memset(cleaned, 0, number);
+    memset(population, 0, sqr(number));
+    memset(users, 0, number*sizeof(afs_int32));
+    memset(groups, 0, number*sizeof(afs_int32));
 
     ownerUser = lastGroup = 0;
     groupOwners = (afs_int32 *) malloc (number*sizeof(afs_int32));
@@ -898,9 +914,12 @@ static int MyBeforeProc (as, arock)
 	    strncpy (cellinfo.hostName[i], serverList->data, MAXHOSTCHARS);
 	    th = gethostbyname(cellinfo.hostName[i]);
 	    if (!th) return UBADHOST;
-	    bcopy(th->h_addr, &cellinfo.hostAddr[i].sin_addr, sizeof(afs_int32));
+	    memcpy(&cellinfo.hostAddr[i].sin_addr, th->h_addr, sizeof(afs_int32));
 	    cellinfo.hostAddr[i].sin_family = AF_INET;
 	    cellinfo.hostAddr[i].sin_port = 0;
+#ifdef STRUCT_SOCKADDR_HAS_SA_LEN
+	    cellinfo.hostAddr[i].sin_len = sizeof(struct sockaddr_in);
+#endif
 	}
 	cellinfo.numServers = i;
 	strcpy (cellinfo.name, lcell);
@@ -990,12 +1009,12 @@ main (argc, argv)
     struct cmd_syndesc *ts;	    /* ptr to parsed command line syntax */
 
     whoami = argv[0];
-    initialize_cmd_error_table();
-    initialize_acfg_error_table();
-    initialize_ktc_error_table();
-    initialize_u_error_table();
-    initialize_pt_error_table();
-    initialize_rxk_error_table();
+    initialize_CMD_error_table();
+    initialize_ACFG_error_table();
+    initialize_KTC_error_table();
+    initialize_U_error_table();
+    initialize_PT_error_table();
+    initialize_RXK_error_table();
 
 #ifdef AFS_NT40_ENV 
     /* initialize winsock */
