@@ -14,13 +14,14 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.9.2.11 2003/01/11 07:48:58 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.16 2002/02/16 18:23:59 shadow Exp $");
 
 #include "../rx/rx_kcommon.h"
 
 #ifdef AFS_HPUX110_ENV
 #include "../h/tihdr.h"
 #include <xti.h>
+#include "../afs/hpux_110.h"
 #endif
 #include "../afsint/afsint.h"
 
@@ -367,11 +368,7 @@ register struct rx_peer *pp;
       pp->ifMTU = RX_REMOTE_PACKET_SIZE;
     }
 #else /* AFS_USERSPACE_IP_ADDR */
-#ifdef AFS_DARWIN60_ENV
-    struct ifaddr *ifad = (struct ifaddr *) 0;
-#else
     struct in_ifaddr *ifad = (struct in_ifaddr *) 0;
-#endif
     struct ifnet *ifn;
 
     /* At some time we need to iterate through rxi_FindIfnet() to find the
@@ -648,25 +645,7 @@ int rxi_GetIFInfo()
     }
    return different;
 }
-#ifdef AFS_DARWIN60_ENV
-/* Returns ifnet which best matches address */
-struct ifnet *
-rxi_FindIfnet(addr, pifad) 
-     afs_uint32 addr;
-     struct ifaddr **pifad;
-{
-  struct sockaddr_in s;
 
-  if (numMyNetAddrs == 0)
-    (void) rxi_GetIFInfo();
-
-  s.sin_family=AF_INET;
-  s.sin_addr.s_addr=addr;
-  *pifad=ifa_ifwithnet((struct sockaddr *)&s);
- done:
-  return (*pifad ?  (*pifad)->ifa_ifp : NULL );
-}
-#else
 /* Returns ifnet which best matches address */
 struct ifnet *
 rxi_FindIfnet(addr, pifad) 
@@ -731,7 +710,6 @@ rxi_FindIfnet(addr, pifad)
  done:
   return (*pifad ?  (*pifad)->ia_ifp : NULL );
 }
-#endif
 #endif /* else AFS_USERSPACE_IP_ADDR */
 #endif /* !SUN5 && !SGI62 */
 
@@ -1024,15 +1002,14 @@ int rxk_ListenerPid; /* Used to signal process to wakeup at shutdown */
 
 #ifdef AFS_SUN5_ENV
 /*
- * Run the listener as a kernel thread.
+ * Run the listener as a kernel process.
  */
 void rxk_Listener(void)
 {
     extern id_t syscid;
     void rxk_ListenerProc(void);
-    if (thread_create(NULL, DEFAULTSTKSZ, rxk_ListenerProc,
-	0, 0, &p0, TS_RUN, minclsyspri) == NULL)
-	osi_Panic("rxk_Listener: failed to start listener thread!\n");
+    if (newproc(rxk_ListenerProc, syscid, 59))
+	osi_Panic("rxk_Listener: failed to fork listener process!\n");
 }
 
 void rxk_ListenerProc(void)
@@ -1048,11 +1025,11 @@ void rxk_Listener(void)
     rxk_ListenerPid = current->pid;
 #endif
 #ifdef AFS_SUN5_ENV
-    rxk_ListenerPid = 1;      /* No PID, just a flag that we're alive */
+    rxk_ListenerPid = curproc->p_pid;
 #endif /* AFS_SUN5_ENV */
 #ifdef AFS_FBSD_ENV
     rxk_ListenerPid = curproc->p_pid;
-#endif /* AFS_FBSD_ENV */
+#endif /* AFS_SUN5_ENV */
 #if defined(AFS_DARWIN_ENV)
     rxk_ListenerPid = current_proc()->p_pid;
 #endif
@@ -1074,9 +1051,6 @@ void rxk_Listener(void)
 	    rxp = rxi_ReceivePacket(rxp, rx_socket, host, port);
 	    AFS_RXGUNLOCK();
 	}
-	if (afs_termState == AFSOP_STOP_RXK_LISTENER)
-	    break;
-
     }
 
 #ifdef RX_ENABLE_LOCKS
@@ -1092,6 +1066,11 @@ void rxk_Listener(void)
 #endif
 #ifdef AFS_SUN5_ENV
     AFS_GUNLOCK();
+#ifdef HAVE_P_COREFILE
+    if (!curproc->p_corefile)  /* newproc doesn't set it, but exit frees it */
+	curproc->p_corefile = refstr_alloc("core");
+#endif
+    exit(CLD_EXITED, 0);
 #endif /* AFS_SUN5_ENV */
 }
 
