@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/up.c,v 1.9 2001/07/12 19:59:27 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/venus/up.c,v 1.11 2002/01/01 18:57:32 shadow Exp $");
 
 /* missing type from C language */
 #define Boolean short
@@ -49,9 +49,7 @@ RCSID("$Header: /cvs/openafs/src/venus/up.c,v 1.9 2001/07/12 19:59:27 shadow Exp
 
 #define MAXACL 400
 
-extern char *index ();
-extern char *rindex ();
-#if !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_FBSD_ENV)
+#if !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_XBSD_ENV)
 extern sys_nerr;
 extern char *sys_errlist[];
 #endif
@@ -191,7 +189,7 @@ Boolean MakeParent(file, owner)
     
     strcpy(parent, file);
     
-    p = rindex(parent, '/');
+    p = strrchr(parent, '/');
     if (!p) {
 	strcpy(parent, ".");
     }
@@ -684,7 +682,7 @@ int isMountPoint( name, blob )
     /*
      * Find rightmost slash, if any.
      */
-    last_component = (char *) rindex(true_name, '/');
+    last_component = (char *) strrchr(true_name, '/');
     if (last_component) {
         /*
          * Found it.  Designate everything before it as the parent directory,
@@ -713,7 +711,7 @@ int isMountPoint( name, blob )
     blob->in_size = strlen(last_component)+1;
     blob->out_size = MAXSIZE;
     blob->out = space;
-    bzero(space, MAXSIZE);
+    memset(space, 0, MAXSIZE);
 
     code = pioctl(parent_dir, VIOC_AFS_STAT_MT_PT, blob, 0);
 
