@@ -38,7 +38,7 @@ static int newVLDB = 1;
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/vol/fssync.c,v 1.5.2.5 2003/02/03 20:46:20 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/vol/fssync.c,v 1.6 2002/02/16 18:24:03 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -252,23 +252,10 @@ static void FSYNC_sync() {
     extern VInit;
     int code;
     int numTries;
-#ifdef AFS_PTHREAD_ENV
-    int tid;
-#endif
 
 #ifndef AFS_NT40_ENV
     signal(SIGPIPE, SIG_IGN);
 #endif
-
-#ifdef AFS_PTHREAD_ENV
-    /* set our 'thread-id' so that the host hold table works */ 
-    MUTEX_ENTER(&rx_stats_mutex);   /* protects rxi_pthread_hinum */
-    tid=++rxi_pthread_hinum;
-    MUTEX_EXIT(&rx_stats_mutex);
-    pthread_setspecific(rx_thread_id_key, (void *)tid);
-    Log("Set thread id %d for FSYNC_sync\n", tid);
-#endif /* AFS_PTHREAD_ENV */
-
     while (!VInit) {
       /* Let somebody else run until level > 0.  That doesn't mean that 
        * all volumes have been attached. */
@@ -334,7 +321,7 @@ static void FSYNC_com(fd)
     int fd;
 {
     byte rc = FSYNC_OK;
-    int n, i, ack = 1;
+    int n, i;
     Error error;
     struct command command;
     int leaveonline;
@@ -520,12 +507,6 @@ defect #2080 for details.
 		vp->specialStatus = VMOVED;
 		VPutVolume_r(vp);
 	    }
-#ifdef AFS_NT40_ENV
-	    send(fd, &rc, 1, 0);
-#else
-	    write(fd, &rc, 1);
-#endif
-	    ack = 0;
 	    if (V_BreakVolumeCallbacks) {
 		Log("fssync: volume %u moved to %x; breaking all call backs\n",
 		    command.volume, command.reason);
@@ -538,12 +519,6 @@ defect #2080 for details.
 	    break;
 	case FSYNC_RESTOREVOLUME:
 	    /* if the volume is being restored, break all callbacks on it*/
-#ifdef AFS_NT40_ENV
-	    send(fd, &rc, 1, 0);
-#else
-	    write(fd, &rc, 1);
-#endif
-	    ack = 0;
 	    if (V_BreakVolumeCallbacks) {
 		VOL_UNLOCK
 		VATTACH_UNLOCK
@@ -558,13 +533,11 @@ defect #2080 for details.
     }
     VOL_UNLOCK
     VATTACH_UNLOCK
-    if (ack) {
 #ifdef AFS_NT40_ENV
-	send(fd, &rc, 1, 0);
+    send(fd, &rc, 1, 0);
 #else
-	write(fd, &rc, 1);
+    write(fd, &rc, 1);
 #endif
-    }
 }
 
 static void FSYNC_Drop(fd)
